/*
  program: ECHO485.C
  use    : echo485 [baudrate]
	   (1) must run on 7188
	   (2) default baudrate is 9600
  function description:
	   (1) echo any data on the 7188's COM2(RS485) to COM4.
	   (2) data come from COM4(end with CR(0x0d)) will be send to
	       COM2(RS485)(data in will be buffered,until receive 0x0d,
	       7188 will send out the buffered data.(if checksum enable,
	       will append checksum(2 bytes) after the data,before 0x0d.)
	   (3) press 'q'(not 'Q') to quit.
	   (4) press 's'(not 'S') to toggle checksum mode.
	       so 'q' and 's' will not be send to 485.
	   (5) default mode is checksum enable.
	   (6) protocal is 1 start bit,8 data bit,none parity,1 stop bit.

   [PC]<---------->[7188-0]<------------------+----->[7000's]
	RS232  (COM4)    (COM2)  RS485        |
		   [7188-1]<------------------+
			 (COM2/COM1) RS485
   NOTE: Use echo485 to (1)monitor data on RS485,(2)send command to 7000's

[10/11/1999]
Ver.1.03
 change putchar() to putch4()
*/
#include<stdlib.h>
#include<stdio.h>
#include<conio.h>
#include<string.h>
#include"..\lib\8000.h"

unsigned no;
unsigned char buf[80];
unsigned char OutBuf[80];
int idx=0;
int outidx=0;

main(int argc,char *argv[])
{ long baud=9600;
  int quit=0;
  int data;
  int i;
  int checksum=0;
  int fSearch=0;
  int Addr;
  unsigned wait=0;

  if(!Is8000()){
	 puts("\necho485 must run on 8000!");
     return;
  }
  if(argc>1){
     baud=atol(argv[1]);
     if(baud<=0) baud=9600;
  }
  Print("\n\rEcho485 Ver. 1.03"
	"\n\rUse COM2(RS485) connect to 7000's"
	"\n\recho485 [baudrate] -->  default:9600"
	"\n\r's' to toggle checksum ON(1)/OFF(0)"
	"\n\rCTRL_S to search 7000's modules *** NEW ***"
	"\n\r'q' to quit to ROM-DOS"
	"\n\rCurrent mode:baudrate=%ld,checksum=%d\n\r",baud,checksum);
  InstallCom2(baud,8,0,1);
  Init5DigitLed();
  Show5DigitLed(1,15);
  Show5DigitLed(2,12);
  Show5DigitLed(3,4);
  Show5DigitLed(4,8);
  Show5DigitLed(5,5);
  OutBuf[0]=0;

  while(!quit){
	 if(fSearch){
	if(!wait){
	   sprintf(OutBuf,"$%02XM",Addr);
	   outidx=4;
	   wait=1;
	   Print("\r%02X",Addr);
	   goto SendCmd;
	}
	else {
	   wait++;
	   if(wait>=1000) {
		  wait=0;
		  Addr++;
		  if(Addr>=256){
		 fSearch=0;
		 Print("\n");
		  }
	   }
	}
     }
	 if(Kbhit()){
	data=Getch();
	if(data=='q') quit=1;
	else if(data==19){ /* CTRL_S */
	     fSearch=1;
	     Addr=0;
	     continue;
	}
	else if(data=='s'){
	   checksum=!checksum;
	   Print("CheckSum=");
	   Putch('0'+checksum);
	   Print("\n");
	}
	else if(data=='\b'){
		  if(outidx){
		 outidx--;
		 Putch(data);
		  }
	}
	else {
	   Putch(data);
	   if(data=='\r'){
		  unsigned char sum;
		  Putch('\n');
		  if(outidx) OutBuf[outidx]=0;
		  if(!strnicmp("baud ",OutBuf,5)){
		     long baud2=atol(OutBuf+5);
		     SetBaudrate2(baud2);
		     Print("Change baudrate to %ld\n\r",baud2);
		  }
		  else {
SendCmd:
		    sum=0;
   //	      Set485DirToTransmit(2);
		    for(i=0;i<outidx;i++){
			ToCom2(OutBuf[i]);
			if(checksum) sum+=OutBuf[i];
		    }
		    if(checksum){
		       ToCom2(hex_to_ascii[sum>>4]);
		       ToCom2(hex_to_ascii[sum&0xf]);
		    }
		    ToCom2('\r');
		  }
		  outidx=0;
//		  WaitTransmitOver2();
//		  Set485DirToReceive(2);
	   }
	   else {
		  OutBuf[outidx++]=data;
	   }
	}
	 }
	 if(IsCom2()){
	data=ReadCom2();
	Putch(data);
	if(data=='\r'){
	   Putch('\n');
	   buf[idx]=0;
	   if(checksum){
		  unsigned char sum=0;
		  int i;
		  for(i=0;i<idx-2;i++)
		 sum+=buf[i];
		  if(buf[i] != hex_to_ascii[sum>>4] ||
		 buf[i+1] != hex_to_ascii[sum&0x0f]) {
		 Print("Check sum error,must be %02X",sum);
		  }
	   }
	   idx=0;
	   if(wait){
		  Print("wait=%u\n",wait);
		  wait=0;
		  Addr++;
		  if(Addr>=256){
		 fSearch=0;
		 Print("\n");
		  }
	   }
	}
	else {
	   buf[idx++]=data;
	   if(idx>=79) idx=78;
	}
	 }
  }
  RestoreCom2();
}