/*
XS_NTP.c: Time Synchronization using NTP

Compiler: BC++ 3.1, 
          Turbo C++ 1.01(3.01)

Compile mode: Large

Project: XS_NTP.c
         ..\lib\7188el.Lib
         ..\lib\tcpip32.lib
         ..\lib\FW_nnnnn.LIB
         
Hordware: I-7188EX

Active servers: http://www.eecis.udel.edu/~mills/ntp/clock1a.html  

This demo is used to synchronize the time of a computer client
or server to another server or reference time source through UDP protocol,
Type13, 14 (Timestamp Request and Timestamp Reply Message)

The timestamp values are in milliseconds from midnight UT (Greenwich mean time)
    Orig                   Recv  Xmit                    RecvReply
    |----------------------->|    |------------------------->|
    |<----------------------- RTT -------------------------->|
        RTT(round-trip time), Recv=Xmit

   diff=RTT/2-(Recv-Orig)=(RecvReply-Orig)/2-(RecvReply-Xmit)
   diff (It's the time difference between the 7188E and Time server)

Time server:
    time.stdtime.gov.tw   220.130.158.52                            
    nets.org.sg           203.117.180.36
    Active Timer servers: http://www.eecis.udel.edu/~mills/ntp/clock1a.html

[Dec 19, 2008] by Liam
[Aug,02,2011] by Nicholas
*/

#include<conio.h>
#include<stdio.h>
#include "..\lib\8000e.h"
#include "..\lib\Tcpip32.h"
#include "..\lib\MFW.h"

#define BUFSIZE 1460    // read/write buffer size
#define TaiwanTime 8    // GMT+ 8.00
#define SNTP_PORT 123

void ShowSocketStatus(void);
char *PublicIP="nets.org.sg";
char IP_Address[20];
unsigned long ulIP;
unsigned char GsIP[4];

int DN_to_IP(unsigned char *sDN,unsigned char *sIP);
void DoUdpNTP(int skt);

UDP_SOCKET UdpNTP={ //UDP client
    -1,         //int   socket;
    0,          //unsigned MyPort;
    "203.117.180.36",   //char *RemoteIp; //nets.org.sg 
    SNTP_PORT,  //unsigned RemotePort;
    1,          //int fNonBlock;
    0,          //int fEnableBroadcast;
    0,          //int status; 
    DoUdpNTP    //void (*CallBackFun)(int skt);
};

unsigned long ltime1;

char *GetModuleName(void)   /* User's program must support this function for UDP search */ 
{
    return "I-7188EX-MFW" ;
}
/************************************************************************/
char *GetAliasName(void)    /* User's program must support this function for UDP search */ 
{
    return "test1" ;
} 

#define DNS1    "10.0.0.103"
#define DNS2    "10.0.0.1"

unsigned char MyIp[4];
unsigned char NewMask[4];
unsigned char NewGateway[6];

void RequestsyncTime(void); 
void XS_UserInit(int argc,char *argv[])
{
    extern int bAcceptBroadcast;
    extern unsigned long ACKDELAY;
    // extern struct NETDATA netdata[];
    extern long MAXTXTOUT;
    void UserLoop(void);

    int i, port;
    unsigned char cIP_Net[10];
    
    InitLib();
            
    for(i=1; i<argc; i++)
	{   // Process of the arguments
        if(argv[i][0]=='/')
		{
            switch(argv[i][1])
			{
                case 'i': case 'I':
                    // set the IP address of the remote server.
                    UdpNTP.RemoteIp=argv[i]+2;
                    Print("[remote IP=%s]",UdpNTP.RemoteIp);
                    break;
					
                case 'p': case 'P':
                    // set the TCP port of the remote server.
                    if(1==sscanf(argv[i]+2,"%i",&port)){
                        UdpNTP.RemotePort=port;
                        Print("[remote port=%u(%x)]",UdpNTP.RemotePort,UdpNTP.RemotePort);
                    }
                    break;
					
                case 'd': case 'D':
                    bUseDhcp=1;
                    break;  
            }
        }
    }
    
    bAcceptBroadcast=0;
    ACKDELAY=200;

    XS_AddSystemLoopFun(UserLoop);
    
    // for using DHCP, Set the IP address/MASK to 0 before call Ninit().
    if(bUseDhcp)
	{
        extern struct NETDATA netdata[];
        // for using DHCP, Set the IP address/MASK to 0 before call Ninit().
        *(long *)netdata[0].Iaddr.c=
        *(long *)netdata[0].Imask.c=
        *(long *)netdata[1].Iaddr.c=
        *(long *)netdata[1].Imask.c=0L;
        // DhcpLeaseTime=xxxx;  // by default DhcpLeaseTime=0,
        // DhcpLeaseTime=20;    // that is the time is assigned by the DHCP server.
                                // the time unit is second.
        Install_DHCP(); // must call Install_DHCP() for DHCP function to work.
    }
    else
	{  
        // add DNS server here
        AddDomainNameServer(DNS1);
  #ifdef DNS2
        AddDomainNameServer(DNS2);
  #endif    
    }   

    /* Add UDP telnet client */
    XS_AddUdpSocket(&UdpNTP);
    
    XS_AddSystemLoopFun(XS_SocketLoopFun);  
    XS_StartSocket();

    MAXTXTOUT=6000UL; 
    ltime1=GetTimeTicks();
  
    //SetUserTimer(1);
    DT2_AddTimer(30000, 1, RequestsyncTime);
  
    Delay(20);
    Print("DnsIpNo=%d, DNS1=%s, DNS2=%s \r\n", DnsIpNo, DnsIp[0], DnsIp[1]);
    Print("Currnt IP address=%d.%d.%d.%d\r\n", netconf[nets[0].confix].Iaddr.c[0], netconf[nets[0].confix].Iaddr.c[1], netconf[nets[0].confix].Iaddr.c[2], netconf[nets[0].confix].Iaddr.c[3]);
    DN_to_IP(PublicIP,IP_Address);
    UdpNTP.RemoteIp=IP_Address;       
    Print("%s = %s\r\n", PublicIP, IP_Address); 
}

void XS_UserEnd(void)
{
    XS_StopSocket();
    DisableWDT();
}

unsigned status_8019as=0;
int Socket0=-1; 
extern int errno;
long st,old_dt;
unsigned char TestData='0'; 

char Command[100];
int idx=0;

void UserLoop(void)
{ 
    long tmpt;
    int key, err;

    RefreshWDT();
  
    if(Kbhit())
	{
        switch((key=Getch()))
		{ 
			//Check whether any data was inputted to command port or not
			//Command port of  I7188EX is COM1
			case 0x73: //'s'
				GetMask(NewMask);
				Print("Current SubMask IP:%d:%d:%d:%d\r\n", NewMask[0], NewMask[1], NewMask[2], NewMask[3]);
				GetGateway(NewGateway);
				Print("Current Gateway IP:%d:%d:%d:%d\r\n", NewGateway[0], NewGateway[1], NewGateway[2], NewGateway[3]); 
				break;
				
            case 27: //Press 'Esc' key to quit the program
                QuitMain=1;
                break;
				
            default: //The other data come-in
                Command[idx]=key;
                if(idx<99) idx++; //Maximum characters of a string ==>99
					Putch(key);
                if(key=='\r')
				{ 
                    //The terminal character of a string is '\r' (Press 'Enter' key)
                    Command[idx]=0;
                    Putch('\n');
                    if(Command[0]=='/')
					{
						// '/' denotes that the inputted string was a command 
                        if(Command[1]=='Q')
						{
							//Break the TCP connection
                        }                       
                        else 
						{   //unknow command
                            Print("Unsupport command\r\n");
                        }
                    }
                    else 
					{
                        Print("[do nothing]\r\n");
                    }
                    idx=0;
                }
                break;
        }
    }
}

void RequestsyncTime(void)
{
    Print("Status=%d\r\n",UdpNTP.status);
    if(UdpNTP.status>0)
        synctime_send(UdpNTP.socket);
}

char buf[BUFSIZE];
void main(int argc, char *argv[])
{
    XS_main(argc, argv);    /* call the XS library main function. */
}

#define ESTABLISHED   1     /* state machine states, see handbook */
#define FINWAIT_1     2
#define FINWAIT_2     3
#define CLOSED_WAIT   4
#define TIMEWAIT      5
#define LAST_ACK      6
#define CLOSED        7
#define SYN_SENT      8
#define SYN_RECEIVED  9
#define LISTEN       16

void ShowSocketStatus(void)
{
    int i;

    Puts("\r\n");
    for(i=0; i<MaxSocketNo; i++)
	{
        Print("[%02d=%02d:", i, connblo[i].state);
        switch(connblo[i].state)
		{
            case ESTABLISHED: Print("ESTABLISHED]"); break;
            case FINWAIT_1: Print("FINWAIT_1]"); break;
            case FINWAIT_2: Print("FINWAIT_2]"); break;
            case CLOSED_WAIT: Print("CLOSED_WAIT]"); break;
            case TIMEWAIT: Print("TIMEWAIT]"); break;
            case LAST_ACK: Print("LAST_ACK]"); break;
            case CLOSED: Print("CLOSED]"); break;
            case SYN_SENT: Print("SYN_SENT]"); break;
            case SYN_RECEIVED: Print("SYN_RECEIVED]"); break;
            case LISTEN: Print("LISTEN]"); break;
            case 0: Print("NOT_USED_YET]"); break;
            default: Print("!!!UnKnown STATE!!!]"); break;
        }
        
        Print(",stat=%x", connblo[i].blockstat);
        if(i & 1) Print("\r\n");
        else Print(" , ");
    }
}
int DN_to_IP(unsigned char *sDN,unsigned char *sIP)
{  
    unsigned char *tmp;
    struct hostent *hptr;

    if((hptr=gethostbyname(sDN)))
    {
        tmp=*(hptr->h_addr_list);
        sprintf(sIP,"%d.%d.%d.%d",tmp[0],tmp[1],tmp[2],tmp[3]);
        return 0; //DN -> IP OK
    }
    else
        return -1;//DN -> IP Faliure
}
void DoUdpNTP(int skt)
{
    struct sockaddr_in sockAddrWhoSent;

    synctime_recv(skt, TaiwanTime);
}
